/*
    Copyright (C) 2012 Modelon AB

    This program is free software: you can redistribute it and/or modify
    it under the terms of the BSD style license.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    FMILIB_License.txt file for more details.

    You should have received a copy of the FMILIB_License.txt file
    along with this program. If not, contact Modelon AB <http://www.modelon.com>.
*/

/** \file fmilib.h
	\brief Include file to be used in client applications of the FMI Library.
	\defgroup fmi_import FMI import library
*/

#ifndef FMILIB_H_
#define FMILIB_H_

#include <stddef.h>
#include "fmilib_config.h"

#include <FMI/fmi_import_context.h>
#include <FMI1/fmi1_import.h>
#include <FMI2/fmi2_import.h>

#endif
